﻿using ELTE.BoardGame.Model;
using System;
using System.Drawing;
using System.Windows.Forms;

namespace ELTE.BoardGame.View
{
    /// <summary>
    /// Táblajáték ablak típusa.
    /// </summary>
    public partial class DrawingForm : Form, IBoardGameView
    {
        #region private fields

        private PlayerData[,] sessionState; // a munkafolyamat állapot
        private Int32 boardWidth; // tábla szélessége
        private Int32 boardHeight; // tábla magassága

        #endregion

        #region Constructors

        /// <summary>
        /// Tic-Tac-Toe ablak létrehozása.
        /// </summary>
        /// <param name="model">A Tic-Tac-Toe játék.</param>
        public DrawingForm()
        {
            this.InitializeComponent();
            
            KeyPreview = true; // elfogjuk a billentyűzeteseményeket
            KeyDown += new KeyEventHandler(TicTacToeForm_KeyDown);
        }

        #endregion

        #region IView events

        /// <summary>
        /// A játék kezdésének eseménye.
        /// </summary>
        public event EventHandler<EventArgs> GameStarted;

        /// <summary>
        /// A játék léptetésének eseménye.
        /// </summary>
        public event EventHandler<BoardEventArgs> GameStepped;

        /// <summary>
        /// Mentés eseménye.
        /// </summary>
        public event EventHandler SaveGame;

        /// <summary>
        /// Betöltés eseménye.
        /// </summary>
        public event EventHandler LoadGame;

        #endregion

        #region IView methods

        /// <summary>
        /// Megjelenített állapot frissítése.
        /// </summary>
        /// <param name="sessionState">A munkafolyamat állapot.</param>
        public void UpdateViewState(PlayerData[,] sessionState)
        {
            this.sessionState = sessionState;
            this.boardWidth = sessionState != null ? sessionState.GetLength(0) : 0;
            this.boardHeight = sessionState != null ? sessionState.GetLength(1) : 0;
            this.panel.Refresh();
        }

        #endregion

        #region Panel event handlers

        /// <summary>
        /// Panel kirajzolásának eseménykezelése.
        /// </summary>
        private void Panel_Paint(object sender, PaintEventArgs e)
        {
            if (this.sessionState == null)
                return;

            Bitmap bitmap = new Bitmap(this.panel.Width, this.panel.Height); // kép a hatékony kirajzoláshoz

            Graphics graphics = Graphics.FromImage(bitmap);
            graphics.Clear(Color.White); // háttér fehérré festése

            // játéktábla rácsai
            Int32 fieldWidth = this.panel.Width / this.boardWidth;
            Int32 fieldHeight = this.panel.Height / this.boardHeight;

            for (Int32 i = 0; i < this.boardWidth; i++)
                graphics.DrawLine(Pens.Black, 0, i * fieldHeight, this.panel.Width, i * fieldHeight);

            for (Int32 j = 0; j < this.boardHeight; j++)
                graphics.DrawLine(Pens.Black, j * fieldWidth, 0, j * fieldWidth, this.panel.Height);

            // a mezőtartalmak
            for (Int32 i = 0; i < this.boardWidth; i++)
                for (Int32 j = 0; j < this.boardHeight; j++)
                {
                    switch (this.sessionState[i, j])
                    { 
                        case PlayerData.PlayerO:
                            graphics.FillEllipse(Brushes.Yellow, i * fieldWidth + fieldWidth / 10, j * fieldHeight + fieldHeight / 10, 8 * fieldWidth / 10, 8 * fieldHeight / 10);
                            break;
                        case PlayerData.PlayerX:
                            graphics.DrawLine(new Pen(Color.Orange, this.panel.Width / 50), i * fieldWidth + fieldWidth / 10, j * fieldHeight + fieldHeight / 10, i * fieldWidth + 9 * fieldWidth / 10, j * fieldHeight + 9 * fieldHeight / 10);
                            graphics.DrawLine(new Pen(Color.Orange, this.panel.Width / 50), i * fieldWidth + 9 * fieldWidth / 10, j * fieldHeight + fieldHeight / 10, i * fieldWidth + fieldWidth / 10, j * fieldHeight + 9 * fieldHeight / 10);
                            break;
                    }
                }

            e.Graphics.DrawImage(bitmap, 0, 0);
        }

        /// <summary>
        /// Panel egérlenyomásának eseménykezelése.
        /// </summary>
        private void Panel_MouseDown(object sender, MouseEventArgs e)
        {
            // megállapítjuk, melyik mezőn van az egér
            Int32 x = e.X * this.boardWidth / this.panel.Width;
            Int32 y = e.Y * this.boardHeight / this.panel.Height;

            this.OnGameStepped(x, y); // minden tevékenység csak egy esemény kiváltása lesz
        }

        #endregion

        #region Form event handlers
        
        /// <summary>
        /// Ablak átmérezetésének eseménykezelése.
        /// </summary>
        private void TicTacToeForm_Resize(object sender, EventArgs e)
        {
            this.panel.Refresh();
        }

        /// <summary>
        /// Billentyű lenyomásának eseménykezelője.
        /// </summary>
        private void TicTacToeForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (!e.Control) // ha nincs lenyomva a Control, nem csinálunk semmit
                return;

            switch (e.KeyCode)
            { 
                case Keys.N: // új játék indítása (Ctrl+N)
                    this.OnGameStarted();
                    break;
                case Keys.L: // betöltés (Ctrl+L)
                    this.OnLoadGame();
                    break;
                case Keys.S: // mentés (Ctrl+S)
                    this.OnSaveGame();
                    break;
            }
            this.panel.Refresh();
        }

        #endregion

        #region Event triggers

        /// <summary>
        /// A játék kezdésének eseménykezelője.
        /// </summary>
        public void OnGameStarted()
        {
            if (this.GameStarted != null)
                this.GameStarted(this, EventArgs.Empty);
        }

        /// <summary>
        /// A játék léptetésének eseménykezelője.
        /// </summary>
        public void OnGameStepped(Int32 column, Int32 row)
        {
            if (this.GameStepped != null)
                this.GameStepped(this, new BoardEventArgs { Column = column, Row = row });
        }

        /// <summary>
        /// Mentés eseménykezelője.
        /// </summary>
        public void OnSaveGame()
        {
            if (this.SaveGame != null)
                this.SaveGame(this, EventArgs.Empty);
        }

        /// <summary>
        /// Betöltés eseménykezelője.
        /// </summary>
        public void OnLoadGame()
        {
            if (this.LoadGame != null)
                this.LoadGame(this, EventArgs.Empty);
        }

        #endregion
    }
}
